package com.ccwangluo.ccim.audio;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.util.Log;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.room.Room;

import com.ccwangluo.ccim.listener.ResultEventCallback;
import com.ccwangluo.ccim.modle.User;
import com.ccwangluo.ccim.util.ErrorCode;
import com.github.helloiampau.janus.generated.Bundle;
import com.github.helloiampau.janus.generated.Janus;
import com.github.helloiampau.janus.generated.JanusCommands;
import com.github.helloiampau.janus.generated.JanusData;
import com.github.helloiampau.janus.generated.JanusError;
import com.github.helloiampau.janus.generated.JanusEvent;
import com.github.helloiampau.janusclientsdk.JanusConfImpl;
import com.github.helloiampau.janusclientsdk.JanusDelegate;
import com.github.helloiampau.janusclientsdk.JanusFactory;
import com.github.helloiampau.janusclientsdk.rtc.AppRTCAudioManager;
import com.github.helloiampau.janusclientsdk.rtc.MediaBundle;
import com.google.gson.Gson;
import com.karumi.dexter.Dexter;
import com.karumi.dexter.MultiplePermissionsReport;
import com.karumi.dexter.PermissionToken;
import com.karumi.dexter.listener.PermissionRequest;
import com.karumi.dexter.listener.multi.MultiplePermissionsListener;

import org.webrtc.AudioTrack;
import org.webrtc.DataChannel;
import org.webrtc.EglBase;
import org.webrtc.VideoTrack;

import java.nio.ByteBuffer;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

public class JanusService extends JanusDelegate {

    private Janus _janus;
    private EglBase _rootEgl;
    private User user;
    private Integer roomId;
    private Boolean isJoin = false;
    private AppRTCAudioManager audioManager;
    private  ResultEventCallback<Void> callback;

    public JanusService(Context context, Integer roomId, User user, ResultEventCallback<Void> callback) {
        audioManager = AppRTCAudioManager.create(context.getApplicationContext());
        this.user = user;
        this.roomId = roomId;
        this.callback = callback;

    }

    public void start(Context context, JanusConfImpl conf) {
        Dexter.withContext(context).withPermissions(Manifest.permission.RECORD_AUDIO).withListener(new MultiplePermissionsListener() {
            @Override
            public void onPermissionsChecked(MultiplePermissionsReport report) {
                JanusFactory factory = new JanusFactory();
                _janus = factory.create(conf,JanusService.this , context.getApplicationContext());
                _rootEgl = factory.rootEgl();
                _janus.init();
                if (audioManager != null) {
                    audioManager.start((selectedAudioDevice, availableAudioDevices) -> {

                            }
                    );
                }
            }

            @Override
            public void onPermissionRationaleShouldBeShown(List<PermissionRequest> permissions, PermissionToken token) {
                callback.onFailed(ErrorCode.ERROR_PERMISSION_DENIE);
            }
        }).check();
    }

    public void stop() {
        this._janus.close();
    }

    public void hangup() {
        this._janus.hangup();
    }


    @Override
    public void onClose() {
        if (audioManager != null) {
            audioManager.stop();
        }
    }

    @Override
    public void onError(JanusError error, Bundle context) {

    }

    @Override
    public void onReady() {
        Bundle payload = Bundle.create();
        payload.setInt("room", roomId);
        payload.setString("display", new Gson().toJson(user));
        this._janus.dispatch(JanusCommands.JOIN, payload);
    }

    @Override
    public void onHangup(String reason) {

    }

    @Override
    public void onMediaChanged(MediaBundle media) {

    }

    @Override
    public void onDatachannelMessage(DataChannel.Buffer buffer) {

    }

    @Override
    public String onOffer(String sdp) {
        return sdp;
    }

    @Override
    public String onAnswer(String sdp) {
        return sdp;
    }

    @Override
    public void onEvent(JanusEvent event, Bundle context) {
        JanusData data = event.data();
        if (data.getString("janus", "").equals("error") && context.getString("command", "").equals(JanusCommands.JOIN)) {
            System.out.println("加入失败");
            if (callback!= null) {
                callback.onFailed(ErrorCode.JOIN_AUDIO_FAILED);
            }
            return;
        }

        if (data.getString("videoroom", "").equals("joined")) {
            if (!isJoin) {
                isJoin = true;
                Bundle payload = Bundle.create();
                payload.setBool("audio", true);
                payload.setBool("video", false);
                payload.setBool("data", false);
                this._janus.dispatch(JanusCommands.PUBLISH, payload);
                System.out.println("加入成功");
                List<JanusData> publishers = data.getList("publishers");
                subscribePublishers(publishers);
                if (callback!= null) {
                    callback.onSuccess(null);
                }
            }
            return;
        }

        List<JanusData> publishers = data.getList("publishers");
        if(publishers.size() > 0) {
            subscribePublishers(publishers);
            return;
        }
    }

    private void subscribePublishers(List<JanusData> publishers) {
        if(publishers.size() > 0) {
            for(JanusData publisher : publishers) {
                Bundle payload = Bundle.create();
                payload.setInt("room", roomId);
                payload.setInt("feed", publisher.getInt("id", -1));
//                payload.setInt("feed", publisher.getString("display", ""));

//                Publisher pub = new Publisher(publisher);
//                publishersMap.put(pub.handleId, pub);

                this._janus.dispatch(JanusCommands.SUBSCRIBE, payload);
            }
        }
    }


}
